@@ -12,7 +12,7 @@ from httpx import Cookies
 from .activation_bytes import get_activation_bytes as get_ab
 from .aescipher import AESCipher, detect_file_encryption
 from .exceptions import FileEncryptionError, AuthFlowError, NoRefreshToken
-from .login import login
+from .login import login, external_login
 from .register import deregister as deregister_
 from .register import register as register_
 from .utils import test_convert
@@ -187,6 +187,7 @@ class Authenticator(httpx.Auth):
 
     Attributes:
         access_token (:obj:`str`, :obj:`None`):
+        activation_bytes (:obj:`str`, :obj:`None`):
         adp_token (:obj:`str`, :obj:`None`):
         crypter (:class:`~audible.aescipher.AESCipher`, :obj:`None`):
         customer_info (:obj:`dict`, :obj:`None`):
@@ -202,6 +203,7 @@ class Authenticator(httpx.Auth):
     """
 
     access_token: Optional[str] = None
+    activation_bytes: Optional[str] = None
     adp_token: Optional[str] = None
     crypter: Optional[AESCipher] = None
     customer_info: Optional[Dict] = None
@@ -316,7 +318,8 @@ class Authenticator(httpx.Auth):
                    register: bool = False,
                    captcha_callback: Optional[Callable[[str], str]] = None,
                    otp_callback: Optional[Callable[[], str]] = None,
-                   cvf_callback: Optional[Callable[[], str]] = None
+                   cvf_callback: Optional[Callable[[], str]] = None,
+                   approval_callback: Optional[Callable[[], Any]] = None
                    ) -> "Authenticator":
         """Instantiate a new Authenticator with authentication data from login.
 
@@ -329,11 +332,12 @@ class Authenticator(httpx.Auth):
                 instance for the marketplace to login.
             register: If ``True``, register a new device after login.
             captcha_callback: A custom callback to handle captcha requests
-                during login
+                during login.
             otp_callback: A custom callback to handle one-time password
                 requests during login.
             cvf_callback: A custom callback to handle verify code requests
                 during login.
+            approval_callback: A custom Callable for handling approval alerts.
 
         Returns:
             An :class:`~audible.auth.Authenticator` instance.
@@ -346,7 +350,8 @@ class Authenticator(httpx.Auth):
             password=password,
             captcha_callback=captcha_callback,
             otp_callback=otp_callback,
-            cvf_callback=cvf_callback)
+            cvf_callback=cvf_callback,
+            approval_callback=approval_callback)
 
         logger.info(f"logged in to Audible as {username}")
 
@@ -356,6 +361,39 @@ class Authenticator(httpx.Auth):
 
         return auth
 
+    @classmethod
+    def from_login_external(cls,
+                            locale: Union[str, "Locale"],
+                            register: bool = False,
+                            login_url_callback: Optional[Callable[[str], str]] = None
+                           ) -> "Authenticator":
+        """Instantiate a new Authenticator from login with external browser.
+
+        .. versionadded:: v0.5.1
+
+        Args:
+            locale: The ``country_code`` or :class:`audible.localization.Locale`
+                instance for the marketplace to login.
+            register: If ``True``, register a new device after login.
+            login_url_callback: A custom Callable for handling login with 
+                external browsers.
+
+        Returns:
+            An :class:`~audible.auth.Authenticator` instance.
+        """
+        auth = cls()
+        auth.locale = locale
+
+        auth.re_login_external(login_url_callback=login_url_callback)
+
+        logger.info("logged in to Audible.")
+
+        if register:
+            auth.register_device()
+            logger.info("registered Audible device")
+
+        return auth
+
     def auth_flow(self,
                   request: httpx.Request
                   ) -> Generator[httpx.Request, httpx.Response, None]:
@@ -442,6 +480,12 @@ class Authenticator(httpx.Auth):
                 indent: int = 4,
                 set_default: bool = True,
                 **kwargs) -> None:
+        """Save authentication data to file.
+        
+        .. versionadded:: 0.5.1
+        
+           Save activation bytes to auth file
+       """
 
         if not (filename or self.filename):
             raise ValueError("No filename provided")
@@ -466,7 +510,8 @@ class Authenticator(httpx.Auth):
             "device_info": self.device_info,
             "customer_info": self.customer_info,
             "expires": self.expires,
-            "locale_code": self.locale.country_code
+            "locale_code": self.locale.country_code,
+            "activation_bytes": self.activation_bytes
         }
         json_body = json.dumps(body, indent=indent)
 
@@ -488,7 +533,7 @@ class Authenticator(httpx.Auth):
                 encryption=encryption,
                 indent=indent)
 
-        logger.info(f"saved data to file {filename}")
+        logger.info(f"saved data to file {target_file}")
 
         if set_default:
             self.filename = target_file
@@ -502,7 +547,8 @@ class Authenticator(httpx.Auth):
                  password: str,
                  captcha_callback: Optional[Callable[[str], str]] = None,
                  otp_callback: Optional[Callable[[], str]] = None,
-                 cvf_callback: Optional[Callable[[], str]] = None) -> None:
+                 cvf_callback: Optional[Callable[[], str]] = None,
+                 approval_callback: Optional[Callable[[], Any]] = None) -> None:
 
         login_device = login(
             username=username,
@@ -512,7 +558,30 @@ class Authenticator(httpx.Auth):
             market_place_id=self.locale.market_place_id,
             captcha_callback=captcha_callback,
             otp_callback=otp_callback,
-            cvf_callback=cvf_callback)
+            cvf_callback=cvf_callback,
+            approval_callback=approval_callback)
+
+        self._update_attrs(**login_device)
+
+    def re_login_external(self,
+                          login_url_callback: Optional[Callable[[str], str]] = None
+                         ) -> None:
+        """Re-login with a external browser and refreshs the access token.
+
+        .. versionadded:: v0.5.1
+
+        Args:
+            locale: The ``country_code`` or :class:`audible.localization.Locale`
+                instance for the marketplace to login.
+            register: If ``True``, register a new device after login.
+            login_url_callback: A custom Callable for handling login with 
+                external browsers.
+        """
+        login_device = external_login(
+            country_code=self.locale.country_code,
+            domain=self.locale.domain,
+            market_place_id=self.locale.market_place_id,
+            login_url_callback=login_url_callback)
 
         self._update_attrs(**login_device)
 
@@ -552,16 +621,38 @@ class Authenticator(httpx.Auth):
     def get_activation_bytes(self,
                              filename: Optional[
                                  Union["pathlib.Path", str]] = None,
-                             extract: bool = True) -> Union[str, bytes]:
+                             extract: bool = True,
+                             force_refresh: bool = False) -> Union[str, bytes]:
         """Get Activation bytes from Audible
 
         :param filename: [Optional] filename to save the activation blob
         :type filename: string, pathlib.Path
         :param extract: [Optional] if True, returns the extracted activation
                         bytes otherwise the whole activation blob
-        :type extract: bool
+        :type force_refresh: [Optional] if True, existing activation bytes in 
+                             auth file will be ignored and new activation bytes 
+                             will be requested from server.
+
+        .. versionadded:: 0.5.1
+        
+           The ``force_refresh`` argument. Fetched activation bytes are now 
+           stored to `Authententicator.activation_bytes`.
+
         """
-        return get_ab(self, filename, extract)
+        if not force_refresh and extract and self.activation_bytes is not None:
+            logger.debug(f"Activation bytes already fetched. Returned saved one.")
+            return self.activation_bytes
+
+        logger.debug("Fetch activation blob from server now.")
+        ab = get_ab(self, filename, extract)
+        
+        if extract:
+            logger.debug("Extract activation bytes from blob and store value"
+                         "activation_bytes attribute.")
+            logger.debug(f"Found activation bytes: {ab}")
+            self.activation_bytes = ab
+
+        return ab
 
     def user_profile(self) -> Dict:
         return user_profile(
