@@ -1,5 +1,8 @@
-from datetime import datetime, timedelta
+import base64
 import io
+import json
+import secrets
+from datetime import datetime, timedelta
 from typing import Any, Callable, Dict, Optional
 from urllib.parse import urlencode, parse_qs
 
@@ -9,8 +12,8 @@ import httpx
 
 from .metadata import encrypt_metadata, meta_audible_app
 
-USER_AGENT = "Mozilla/5.0 (iPhone; CPU iPhone OS 12_3_1 like Mac OS X) " \
-             "AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148"
+USER_AGENT = ("Mozilla/5.0 (iPhone; CPU iPhone OS 14_1 like Mac OS X) "
+             "AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148")
 
 
 def default_captcha_callback(captcha_url: str) -> str:
@@ -49,6 +52,21 @@ def default_approval_alert_callback() -> None:
     input("Please press enter when you approve the notification.")
 
 
+
+def default_login_url_callback(url: str) -> str:
+    """Helper function for login with external browsers."""
+    print("Please copy the following url and insert it in a webbrowser of "
+          "your choice:")
+    print("\n" + url + "\n")
+    print("Now you have to login with your Amazon credentials. After submit "
+          "your username and password you have to do this a second time "
+          "and solving a captcha before sending the login form.\n")
+    print("After login, your browser will show you a error page (not found). "
+          "Do not worry about this. It has to be like this. Please copy the "
+          "url from the address bar in your browser now.\n")
+
+    return input("Please insert the copied url (after login):\n")
+
 def get_soup(resp):
     return BeautifulSoup(resp.text, "html.parser")
 
@@ -93,6 +111,29 @@ def build_oauth_url(
     return f"https://www.amazon.{domain}/ap/signin?{urlencode(oauth_params)}"
 
 
+def build_init_cookies() -> Dict[str, str]:
+    """Build initial cookies to prevent captcha in most cases."""
+    frc = secrets.token_bytes(313)
+    frc = base64.b64encode(frc).decode("ascii").rstrip("=")
+
+    map_md = {
+        "device_user_dictionary": [],
+        "device_registration_data": {
+            "software_version": "33501644"
+        },
+        "app_identifier": {
+            "app_version": "3.35.1",
+            "bundle_id": "com.audible.iphone"
+        }
+    }
+    map_md = json.dumps(map_md)
+    map_md = base64.b64encode(map_md.encode()).decode().rstrip("=")
+
+    amzn_app_id = "MAPiOSLib/6.0/ToHideRetailLink"
+
+    return {"frc": frc, "map-md": map_md, "amzn-app-id": amzn_app_id}
+
+
 def check_for_captcha(soup: BeautifulSoup) -> bool:
     """Checks a Amazon login page for a captcha form."""
     captcha = soup.find("img", alt=lambda x: x and "CAPTCHA" in x)
@@ -150,7 +191,8 @@ def login(
     market_place_id: str,
     captcha_callback: Optional[Callable[[str], str]] = None,
     otp_callback: Optional[Callable[[], str]] = None,
-    cvf_callback: Optional[Callable[[], str]] = None
+    cvf_callback: Optional[Callable[[], str]] = None,
+    approval_callback: Optional[Callable[[], Any]] = None
 ) -> Dict[str, Any]:
     """Login to Audible by simulating an Audible App for iOS.
     
@@ -164,8 +206,10 @@ def login(
             If ``None`` :func:`default_captcha_callback` is used.
         otp_callback: A custom Callable for providing one-time passwords.
             If ``None`` :func:`default_otp_callback` is used.
-        cvf_callback: A custom Callback for providing the answer for a CVF code.
+        cvf_callback: A custom Callable for providing the answer for a CVF code.
             If ``None`` :func:`default_cvf_callback` is used.
+        approval_callback: A custom Callable for handling approval alerts.
+            If ``None`` :func:`default_approval_alert_callback` is used.
     
     Returns:
         An ``access_token`` with ``expires`` timestamp and the 
@@ -176,8 +220,14 @@ def login(
     sign_in_url = amazon_url + "/ap/signin"
     cvf_url = amazon_url + "/ap/cvf/verify"
 
-    default_headers = {"User-Agent": USER_AGENT}
-    session = httpx.Client(headers=default_headers)
+    default_headers = {
+        "User-Agent": USER_AGENT,
+        "Accept-Language": "en-US",
+        "Accept-Encoding": "gzip"
+    }
+    init_cookies = build_init_cookies()
+
+    session = httpx.Client(headers=default_headers, cookies=init_cookies)
 
     while "session-token" not in session.cookies:
         session.get(amazon_url)
@@ -189,7 +239,7 @@ def login(
     login_inputs = get_inputs_from_soup(oauth_soup)
     login_inputs["email"] = username
     login_inputs["password"] = password
-    
+
     metadata = meta_audible_app(USER_AGENT, amazon_url)
     login_inputs["metadata1"] = encrypt_metadata(metadata)
 
@@ -266,7 +316,11 @@ def login(
 
     # check for approval alert
     while check_for_approval_alert(login_soup):
-        default_approval_alert_callback()
+        if approval_callback:
+            approval_callback()
+        else:
+            default_approval_alert_callback()
+
         url = login_soup.find_all("a", class_="a-link-normal")[1]["href"]
 
         login_resp = session.get(url)
@@ -286,3 +340,44 @@ def login(
         "website_cookies": website_cookies,
         "expires": expires
     }
+
+
+def external_login(
+    country_code: str,
+    domain: str,
+    market_place_id: str,
+    login_url_callback: Optional[Callable[[str], str]] = None
+) -> Dict[str, Any]:
+    """Gives the url to login with external browser and prompt for result.
+    
+    Args:
+        country_code: The country code for the Audible marketplace to login.
+        domain: domain: The top level domain for the Audible marketplace to login.
+        market_place_id: The id for the Audible marketplace to login.
+        login_url_callback: A custom Callable for handling login with external 
+            browsers. If ``None`` :func:`default_login_url_callback` is used.
+
+    Returns:
+        An ``access_token`` with ``expires`` timestamp from the 
+        authorized Client.
+    """
+    oauth_url = build_oauth_url(country_code, domain, market_place_id)
+
+    if login_url_callback:
+        response_url = login_url_callback(oauth_url)
+    else:
+        response_url = default_login_url_callback(oauth_url)
+
+    response_url = httpx.URL(response_url)
+    parsed_url = parse_qs(response_url.query.decode())
+
+    access_token = parsed_url["openid.oa2.access_token"][0]
+
+    auth_time = parsed_url["openid.pape.auth_time"][0]
+    auth_time = datetime.strptime(auth_time, "%Y-%m-%dT%H:%M:%SZ")
+    expires = (auth_time + timedelta(seconds=3600)).timestamp()
+
+    return {
+        "access_token": access_token,
+        "expires": expires
+    }
\ No newline at end of file
